#!/usr/bin/python
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# (c) 2007 Red Hat. Written by skvidal@fedoraproject.org
# preupgrade-cli

import sys
import os
import os.path

sys.path.insert(0,'/usr/share/yum-cli/')
from cli import *
from utils import YumUtilBase
from yum import Errors
import preupgrade
from preupgrade import hrsize
from urlgrabber.grabber import URLGrabError
from output import YumTextMeter
        
class PreUpgradeCli(preupgrade.PreUpgrade, YumUtilBase):
    NAME = 'PreUpgrade' 
    USAGE = '"usage: preupgrade [options] release [path]'

    def __init__(self):
        preupgrade.PreUpgrade.__init__(self)
        YumUtilBase.__init__(self,
                             PreUpgradeCli.NAME,
                             preupgrade.__version__,
                             PreUpgradeCli.USAGE)
        self.main()

    def _valid_entries(self):
        cur_ver = float(self.conf.yumvar['releasever'])    
        print "valid entries include:"
        for rel in self.return_available_versions(cur_ver, beta_ok=True):
            print '   "%s"' % rel

    def getconfig(self):
        # Add util commandline options to the yum-cli ones
        parser = self.getOptionParser() 
        # Parse the commandline option and setup the basics.
        opts = self.doUtilConfigSetup()
        if len(self.cmds) < 1:
            print "please give a release to try to pre-upgrade to"
            self._valid_entries()
            sys.exit(1)
        myrelease = self.cmds[0]
        if not self.release_map.has_key(myrelease):
            self.errorprint("no release named %s available" % myrelease)
            self._valid_entries()
            sys.exit(1)
        return myrelease
    
    def errorprint(self, msg):
        print >> sys.stderr, 'Error: %s' % msg
        
    def main(self):
        myrelease = self.getconfig()
        # Step 1: set up yum
        self.setup(myrelease)

        try:
            self.setup_update_repo()
        except Errors.YumBaseError, e:
            self.errorprint(str(e) + "\n\n" + \
                "This could be caused by a missing network connection or a bad mirror.")
            sys.exit(1)
            
        # setup progress callbacks on the enabled repos
        if sys.stdout.isatty():         
            self.setupProgressCallbacks()
        
        # Step 2: Get installer images
        self.retrieve_treeinfo()

        # let the user ctrl-C :)
        self.instrepo.setInterruptCallback(self.interrupt_callback)
        # setup progress callback for kernel/stage2 "Repo"        
        if sys.stdout.isatty(): 
            self.instrepo.setCallback(YumTextMeter(fo=sys.stdout))
        
        try:
            self.retrieve_critical_boot_files()
        except URLGrabError, e:
            # FIXME: allow retry, give details (e.g. not enough space)
            # buttons: "Quit" "Try again" "Different Mirror" etc.
            self.errorprint(str(e))
            sys.exit(1)

        # We fall back to getting stage2 from instrepo
        stage2url = os.path.join(self.instrepo.urls[0],self.mainimage)
        extra_args = "stage2=" + stage2url
        # Try to fetch it now
        stage2success = False
        try:
            stage2file = self.retrieve_non_critical_files()
            stage2_abs = self.bootpath+"/"+stage2file
            stage2success = True
        except preupgrade.PUError, e:
            message = str(e) + "\n\n"
            message += "The installer can download this file once it starts, but this requires a wired network connection during installation.\n\nIf you do not have a wired network connection available, you should quit now."
            # FIXME: sleep here?
            self.errorprint(message)
                                
        if stage2success:            
            mpath = preupgrade.bootpath_to_anacondapath(stage2_abs,UUID=True)
            if mpath == None:
                # preupgrade.st_dev_to_devicename must have failed. Blarg!
                self.errorprint("Could not determine device name for %s" % stage2_abs)
                # FIXME: tell user where to file a bug and what info to include?
            else:
                extra_args = "stage2=%s" % mpath

        # Step 3: Find out what packages we need to download
        try:
            self.figure_out_what_we_need()
        except Errors.YumBaseError,e:
            self.errorprint(str(e))
            sys.exit(1)
            
        downloadpkgs = self.get_download_pkgs()
        
        self.listPkgs(downloadpkgs, 'Packages we need to download', 'list')
        # print out the size to download, and check available disk space
        cachedir = self.distrepo.basecachedir
        downloadsize = self.get_download_size(downloadpkgs)
        print "Total download size: %s" % hrsize(downloadsize)
        dirstat = os.statvfs(cachedir)
        cachefree = dirstat.f_bsize * dirstat.f_bavail
        if downloadsize > cachefree:
            print "You don't have enough free disk space in %s to download"\
                  "the packages needed to upgrade.\n\n"\
                  "You must free up at least %s before you can continue." % \
                  (cachedir,hrsize(downloadsize-cachefree))
            sys.exit(1)
        
        # Step 4: download the rpms we need
        # confirm with user
        print 'Download packages? '
        if self._promptWanted():
            if not self.userconfirm():
                print 'Exiting on user command'
                sys.exit(1)
        problems = self.downloadPkgs(downloadpkgs) 
        del(self.tsInfo)

        upgradesize = self.get_upgrade_size()
        usrstat = os.statvfs("/usr")
        usrfree = usrstat.f_bsize * usrstat.f_bavail
        if upgradesize > usrfree:
            print "Low disk space detected in %s\n\n"\
                  "We recommend that you free up %s before rebooting, "\
                  "or the upgrade may not run.\n\n"\
                  "Note: If the upgrade fails due to low diskspace, "\
                  "your system will not be damaged." % \
                  ("/usr",hrsize(upgradesize-usrfree))

        # FIXME: Step 5 - generate metadata and test transaction
        # Step 5a: make download dir a proper yum repo
        print "Generating metadata for preupgrade repo"
        comps = self.distrepo.retrieveMD("group")
        self.generate_repodata(cachedir, comps) # TODO: callback?
        # Add repo=PATH to boot commandline 
        repopath = preupgrade.bootpath_to_anacondapath(cachedir, UUID=True)
        extra_args = "repo=%s %s" % (repopath,extra_args)
            
        print "Preparing system to boot into installer"
        self.add_boot_target(extra_args)
        self.change_boot_target()

        print 'All finished. The upgrade will begin when you reboot.'
        # FIXME: Ask and reboot for them?

                        
if __name__ == '__main__':
    if os.getuid() == 0:
        util = PreUpgradeCli()
    else:
        print >> sys.stderr, "You need to be root to run this."
        sys.exit(1)
